import Dexie, { Table } from 'dexie';
import dexieCloud, { DexieCloudTable } from 'dexie-cloud-addon';
import { usePermissions } from 'dexie-react-hooks';
import { populate } from './populate';
import { TodoItem } from './TodoItem';
import { TodoList } from './TodoList';

export class TodoDB extends Dexie {
  // Table accessors are auto-generated by Dexie (from schema below)
  todoLists!: DexieCloudTable<TodoList, 'id'>;
  todoItems!: DexieCloudTable<TodoItem, 'id'>;
  localOpenIds!: Table<boolean, [string, string]>;

  constructor() {
    super('TodoDBCloud2', { addons: [dexieCloud] });
    this.version(12).stores({
      todoLists: `@id`,
      todoItems: `@id, [todoListId+realmId]`,
      localOpenIds: `` // Set of open ids (persisted local state only)
    });
    this.todoLists.mapToClass(TodoList);
    this.on('populate', () => {
      this.on('ready', () => {
        return populate(this);
      });
    });
    // Configure cloud:
    this.cloud.configure({
      unsyncedTables: ['localOpenIds'],
      databaseUrl: process.env.REACT_APP_DBURL!,
      tryUseServiceWorker: true, // true!
      requireAuth: false,
    });
  }
}
